"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.hash = exports.isnegative = exports.ispositive = exports.minlength = exports.maxlength = exports.notnull = exports.notzero = exports.isemail = exports.check = exports.use = exports.ref = exports.prop = exports.Schema = void 0;
var crypto_js_1 = __importDefault(require("crypto-js"));
var Schema = function () {
    return function (target) {
        var isExist = target.prototype.hasOwnProperty('jsonschema');
        if (!isExist) {
            Object.defineProperty(target.prototype, 'jsonschema', {
                value: {
                    type: "Object",
                    properties: {},
                    required: []
                }
            });
        }
        Reflect.defineMetadata('class:schema', target.prototype['jsonschema'], target);
        return target;
    };
};
exports.Schema = Schema;
var defaultValue = { required: true, type: 'string', in: 'formData' };
var prop = function (options) {
    if (options === void 0) { options = defaultValue; }
    return function (target, propertyKey) {
        var t = Reflect.getMetadata("design:type", target, propertyKey.toString());
        var propertyName = "".concat(String(propertyKey));
        var isExist = target.hasOwnProperty('jsonschema');
        if (!isExist) {
            Object.defineProperty(target, 'jsonschema', {
                value: {
                    type: "object",
                    properties: {},
                    required: []
                }
            });
        }
        if (options.required) {
            target['jsonschema'].required.push(propertyName);
        }
        target['jsonschema'].properties[propertyName] = __assign(__assign({}, options), { name: propertyName, description: propertyName, default: propertyName, schema: {
                type: "string"
            } });
        Reflect.defineMetadata('class:schema', target['jsonschema'], target);
    };
};
exports.prop = prop;
var ref = function (ref) {
    return function (target, propertyKey) {
        var propertyName = "".concat(String(propertyKey));
        var isExist = target.hasOwnProperty('jsonschema');
        if (!isExist) {
            Object.defineProperty(target, 'jsonschema', {
                value: {
                    type: "object",
                    properties: {},
                    required: []
                }
            });
        }
        target['jsonschema'].properties[propertyName] = use(ref.constructor);
        Reflect.defineMetadata('class:schema', target['jsonschema'], target);
    };
};
exports.ref = ref;
function use(target) {
    var c = target.prototype;
    return c.jsonschema;
}
exports.use = use;
// validators 
var check = function (conditional, message, options) {
    if (options === void 0) { options = { defaultlength: 0, key: '' }; }
    return function (length, option) {
        if (length === void 0) { length = options.defaultlength; }
        return function (target, propertyKey) {
            var propertyname = String(propertyKey);
            var key = '_' + Date.now() + '_' + options.key + '_valid_' + propertyname;
            Object.defineProperty(target, propertyname, {
                get: function () {
                    return target[key];
                },
                set: function (newVal) {
                    var _a;
                    var reflect = Reflect.getMetadata('class:error', target);
                    if (!reflect) {
                        reflect = {};
                    }
                    if (!conditional(newVal, length)) {
                        reflect[propertyname] = (_a = option === null || option === void 0 ? void 0 : option.message) !== null && _a !== void 0 ? _a : message;
                    }
                    else {
                        if (Object.keys(reflect).includes(propertyname)) {
                            delete reflect[propertyname];
                        }
                    }
                    Reflect.defineMetadata('class:error', reflect, target);
                    target[key] = newVal;
                },
                enumerable: true,
                configurable: true
            });
        };
    };
};
exports.check = check;
exports.isemail = (0, exports.check)(function (newVal) {
    var pattern = new RegExp(/^(([^<>()[\]\.,;:\s@\"]+(\.[^<>()[\]\.,;:\s@\"]+)*)|(\".+\"))@(([^<>()[\]\.,;:\s@\"]+\.)+[^<>()[\]\.,;:\s@\"]{2,})$/i);
    return pattern.test(newVal);
}, 'Email not valid', { defaultlength: 10, key: 'isemail' });
exports.notzero = (0, exports.check)(function (newval) {
    return newval != null && newval != undefined && parseInt(newval !== null && newval !== void 0 ? newval : '0', 10) != 0;
}, 'Not zero value', { defaultlength: 10, key: 'notzero' });
exports.notnull = (0, exports.check)(function (newval) {
    return newval != null && newval != undefined;
}, 'Not null', { defaultlength: 10, key: 'notnull' });
exports.maxlength = (0, exports.check)(function (newval, length) {
    return newval && (newval).length <= length;
}, 'Max length not respected', { defaultlength: 10, key: 'maxlength' });
exports.minlength = (0, exports.check)(function (newval, length) {
    return newval && (newval).length >= length;
}, 'Min length not respected', { defaultlength: 10, key: 'minlength' });
exports.ispositive = (0, exports.check)(function (newval) {
    return newval && parseInt(newval !== null && newval !== void 0 ? newval : 0, 0) >= 0;
}, 'Min length not respected', { defaultlength: 10, key: 'ispositive' });
exports.isnegative = (0, exports.check)(function (newval) {
    return newval && parseInt(newval !== null && newval !== void 0 ? newval : 0, 0) < 0;
}, 'Min length not respected', { defaultlength: 10, key: 'isnegative' });
var hash = function (algo) {
    if (algo === void 0) { algo = 'MD5'; }
    return function (target, propertyKey) {
        var key = '_hash_' + String(propertyKey);
        Object.defineProperty(target, propertyKey, {
            get: function () {
                return target[key];
            },
            set: function (newVal) {
                var _a;
                if (typeof algo === 'string') {
                    switch (algo) {
                        case 'MD5':
                            target[key] = crypto_js_1.default.SHA256(newVal).toString();
                            break;
                        case 'SHA1':
                            target[key] = crypto_js_1.default.SHA1(newVal).toString();
                            break;
                        case 'SHA256':
                            target[key] = crypto_js_1.default.SHA256(newVal).toString();
                            break;
                        case 'SHA512':
                            target[key] = crypto_js_1.default.SHA512(newVal).toString();
                            break;
                        default:
                            target[key] = crypto_js_1.default.SHA256(newVal).toString();
                            break;
                    }
                }
                else if (typeof algo === "function") {
                    target[key] = (_a = algo(newVal)) !== null && _a !== void 0 ? _a : newVal;
                }
                else {
                    target[key] = newVal;
                }
            },
            enumerable: true,
            configurable: true
        });
    };
};
exports.hash = hash;
//# sourceMappingURL=shema.js.map